"""
Copyright (c) 2025-now Martian Bugs All rights reserved.
Build Date: 2025-02-10
Author: Martian Bugs
Description: 服务数据模块采集
"""

from DrissionPage import Chromium

from .customer_service import CustomerService


class Service:
    def __init__(self, browser: Chromium):
        self._browser = browser

        self._customer_service = None

    @property
    def customer_service(self):
        """客服数据模块采集"""

        if self._customer_service is None:
            self._customer_service = CustomerService(self._browser)

        return self._customer_service



from time import sleep

from DrissionPage._pages.mix_tab import MixTab


def pick__custom_date(date: str, page: MixTab):
    """选择自定义日期"""

    day_btn = page.ele('t:button@@text()=日', timeout=3)
    if not day_btn:
        raise RuntimeError('未找到 [日] 按钮')

    year, month, day = date.split('-')

    day_btn.click(by_js=True)

    selector_header = page.ele('c:div.d-datepicker-header', timeout=3)
    year_btn, month_btn = selector_header.eles(
        'c:div.d-datepicker-header-main h6.d-text', timeout=1
    )[:2]

    curr_year = year_btn.text.strip()[:-1]
    curr_month = month_btn.text.strip()[:-1]

    selector_header_icons = selector_header.eles('c:span.d-icon.d-clickable', timeout=1)

    def click__arrow_btn(src: str, dst: str, prev_index: int, next_index: int):
        v_number = int(dst)
        curr_v_number = int(src)
        diff_count = abs(curr_v_number - v_number)
        arrow_btn = None
        if curr_v_number > v_number:
            arrow_btn = selector_header_icons[prev_index]
        else:
            arrow_btn = selector_header_icons[next_index]

        for _ in range(diff_count):
            arrow_btn.click(by_js=True)
            sleep(0.3)

    if curr_year != year:
        click__arrow_btn(year, curr_year, -1, 0)

    if curr_month != month:
        click__arrow_btn(month, curr_month, -2, 1)

    day_number = int(day)
    day_btn = page.ele(
        f't:div@@class^d-datepicker-cell-main@@text()={day_number}', timeout=1
    )
    if not day_btn:
        raise RuntimeError(f'日期选择器中未找到日期 {day}')

    day_btn.click(by_js=True)



"""
客服数据模块数据采集
"""

import json

from DrissionPage import Chromium

from ._utils import pick__custom_date


class Urls:
    performance = 'https://ark.xiaohongshu.com/app-datacenter/customer-performance'


class DataPacketUrls:
    performance__detail = (
        'https://ark.xiaohongshu.com/api/edith/butterfly/data?type=sellerCSUserDataList'
    )


class CustomerService:
    def __init__(self, browser: Chromium):
        self._browser = browser
        self._timeout = 15

    def download__performance__detail(
        self,
        date: str,
        save_path: str,
        save_name: str,
        timeout: float = None,
        download_timeout: float = None,
        open_page=False,
        get_shop_name=False,
    ) -> tuple[str | None, str] | str:
        """
        下载客服绩效数据详情表单文件

        Args:
            download_timeout: 下载超时时间, 默认 120 秒
            open_page: 是否打开页面, 如果为 False 则使用当前激活的页面
            get_shop_name: 是否获取店铺名称, 默认为 False
        Returns:
            - 如果 get_shop_name 为 True, 返回 (店铺名称, 下载的文件路径)
            - 否则仅返回店铺名称
        """

        _timeout = timeout if isinstance(timeout, (int, float)) else self._timeout
        _download_timeout = (
            download_timeout if isinstance(download_timeout, (int, float)) else 120
        )

        page = (
            self._browser.new_tab() if open_page is True else self._browser.latest_tab
        )
        if open_page is True:
            page.listen.start(
                targets=DataPacketUrls.performance__detail,
                method='POST',
                res_type='XHR',
            )
            page.get(Urls.performance)
            if not page.listen.wait(timeout=_timeout):
                raise TimeoutError('首次进入页面获取数据超时, 可能页面访问失败')

        download_btn = page.ele('t:span@@text()=下载数据', timeout=3)
        if not download_btn:
            raise RuntimeError('未找到 [下载数据] 按钮')

        page.listen.start(
            targets=DataPacketUrls.performance__detail, method='POST', res_type='XHR'
        )
        pick__custom_date(date, page)
        packet = page.listen.wait(timeout=_timeout)
        if not packet:
            raise TimeoutError('修改日期后数据获取超时')

        req_data: dict = packet.request.postData
        try:
            req_date = req_data['requestBody']['blockElements'][0]['filterMap']['date']
            if req_date != date:
                raise ValueError('当前日期与统计日期不符')
        except ValueError as e:
            raise e
        except Exception as e:
            raise RuntimeError(f'修改日期之后数据请求参数解析出错: {e}') from e

        page.set.download_path(save_path)
        page.set.download_file_name(save_name)
        download_btn.click(by_js=True)
        mission = page.wait.download_begin(timeout=_download_timeout)
        if not page.wait.downloads_done():
            raise TimeoutError('下载超时')

        result = file_path = mission.final_path

        if get_shop_name is True:

            def get__shop_name():
                """获取店铺名称"""
                user_info = page.session_storage('ark_cache_user_info_v2')
                if not user_info:
                    return
                try:
                    user_info: dict = json.loads(user_info)
                except json.JSONDecodeError:
                    return

                seller_name: str = user_info.get('sellerName')
                return seller_name

            shop_name = get__shop_name()
            result = shop_name, file_path

        if open_page is True:
            page.close()

        return result
